///File : PanelRIEdit.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "DialogItemSearch.hpp"

//------------------------------------------------------------------------------
//-                               DialogItemSearch                                   -
//------------------------------------------------------------------------------
DialogItemSearch::DialogItemSearch( wxWindow* parent, wxWindowID id, ItemSectionDBBase *IRDBManager, const wxString &title, const wxPoint& pos, const wxSize& size, long style ) : wxDialog( parent, id, title, pos, size, style )
{
    // Devis Manager
    m_IRDBManager = IRDBManager;

	// Controls
        // Search
	wxBoxSizer *MainSizer = new wxBoxSizer( wxVERTICAL );

    wxBoxSizer *SearchSizer = new wxBoxSizer( wxHORIZONTAL );

    SearchSizer->Add(new wxStaticText(this, wxID_ANY, _("Search :")), 0, wxALIGN_CENTER | wxALL, 5);

	m_TextSearch = new wxTextCtrl(this, wxID_ANY, _T(""));
	m_TextSearch->SetFocus();
	SearchSizer->Add(m_TextSearch, 1, wxALL, 5);

	MainSizer->Add(SearchSizer, 0, wxEXPAND);


        // List
    m_Table = new wxListView(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxLC_SINGLE_SEL );
    MainSizer->Add(m_Table, 1, wxEXPAND);

    MainSizer->Add(CreateButtonSizer(wxOK | wxCANCEL), 0, wxALL | wxALIGN_RIGHT, 5);

	SetSizer(MainSizer);

	// Events
	Connect( wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED, wxEventHandler(DialogItemSearch::OnValidate), NULL, this);
    m_Table->Connect( wxEVT_LEFT_DCLICK, wxEventHandler(DialogItemSearch::OnValidate), NULL, this);
    m_TextSearch->Connect( wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler(DialogItemSearch::OnSearchText), NULL, this);

    // Table
    FeedTbl();
}

//------------------------------------------------------------------------------
std::string DialogItemSearch::GetItemRef() const
{
    return m_itemRef;
}
//------------------------------------------------------------------------------
void DialogItemSearch::OnValidate(wxEvent& event)
{
    long selIndex = m_Table->GetFirstSelected();

    if(m_Table->GetFirstSelected() != -1)
    {
        std::string itemId = StrWxToStd(m_Table->GetItemText(m_Table->GetFirstSelected()));
        if(m_IRDBManager->ItemExists(itemId)) // Si l'item existe...
        {
            m_itemRef = itemId;
            Close();
        }
        else
        {
            wxMessageBox(_("This item doesn't exist !"), _("Item edit"), wxICON_EXCLAMATION, this);
        }
    }
}
//------------------------------------------------------------------------------
void DialogItemSearch::OnSearchText(wxCommandEvent& event)
{
    FeedTbl();

    // Couleur de fond du texte rouge si rien trouvé, vert sinon.
    if( m_TextSearch->GetValue() == _T(""))
    {
        m_TextSearch->SetBackgroundColour(*wxWHITE);
    }
    else if( m_Table->GetItemCount() > 0 )
    {
        m_TextSearch->SetBackgroundColour(*wxGREEN);
    }
    else
    {
        m_TextSearch->SetBackgroundColour(*wxRED);
    }

    m_TextSearch->Refresh();
}
//------------------------------------------------------------------------------
void DialogItemSearch::FeedTbl()
{
    wxWindowUpdateLocker winLock(m_Table); // Bloque les redraw durant son temps de vie.

    // On prend la ref du dernier séléctioner pour le reséléctioonner, si il est toujours là.
    wxString selItemRef = _T("");
    if(m_Table->GetFirstSelected() != -1)
    {
        selItemRef = m_Table->GetItemText(m_Table->GetFirstSelected());
    }

    m_Table->ClearAll();

    wxString searchStr = m_TextSearch->GetValue().Lower(); // TT en lower ==> Case insensitive

    m_Table->InsertColumn(0, _("Reference"));
    m_Table->InsertColumn(1, _("Name"));
    m_Table->InsertColumn(2, _("Price"));
    m_Table->InsertColumn(3, _("Taxe"));
    m_Table->InsertColumn(4, _("Eco Part"));

    std::vector<std::string> itemList = m_IRDBManager->GetAllItemsList();

    for(int i = 0; i < itemList.size(); i++)
    {
        wxString itemRefwxStr = StrStdToWx(itemList[i]);
        wxString itemNamewxStr = StrStdToWx(m_IRDBManager->GetItemName(itemList[i]));

        if(itemRefwxStr.Lower().Find(searchStr) != wxNOT_FOUND || itemNamewxStr.Lower().Find(searchStr) != wxNOT_FOUND) // Si trouvé...
        {
            long tmp = m_Table->InsertItem(i, itemRefwxStr);

            m_Table->SetItem(tmp, 1, itemNamewxStr);

            m_Table->SetItem(tmp, 2, wxNbToStr(m_IRDBManager->GetItemPriceHT(itemList[i])));

            m_Table->SetItem(tmp, 3, wxNbToStr(m_IRDBManager->GetItemTaxe(itemList[i])));

            m_Table->SetItem(tmp, 4, wxNbToStr(m_IRDBManager->GetItemEcoPart(itemList[i])));
        }
    }

    m_Table->SetColumnWidth(0, wxLIST_AUTOSIZE);
    m_Table->SetColumnWidth(1, wxLIST_AUTOSIZE);
    m_Table->SetColumnWidth(2, wxLIST_AUTOSIZE);
    m_Table->SetColumnWidth(3, wxLIST_AUTOSIZE);
    m_Table->SetColumnWidth(4, wxLIST_AUTOSIZE);

    // Special : selectionne la nouvelle ref (a changé de place...)
    if(selItemRef != _T("") && m_Table->FindItem(-1, selItemRef) != -1)
    {
        m_Table->Select(m_Table->FindItem(-1, selItemRef));
    }
    else if(m_Table->GetItemCount() > 0)
    {
        m_Table->Select(0);
    }
}
//------------------------------------------------------------------------------
