///File : ScrolPanelSections.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "MainPanel.hpp"

#if defined USE_SCROLLED_WINDOW
ScrollPanelItems::ScrollPanelItems( wxWindow* parent, DevisManager *devisManager, wxWindowID id, const wxPoint& pos, const wxSize& size, long style ) : wxVScrolledWindow( parent, id, pos, size, style )
#else
ScrollPanelItems::ScrollPanelItems( wxWindow* parent, DevisManager *devisManager, wxWindowID id, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style )
#endif
{
    m_DevisManager = devisManager;

	m_MainSizer = new wxBoxSizer( wxVERTICAL );

	m_SectionPrice = new wxStaticText(this, wxID_ANY, _T(""));
	m_MainSizer->Add(m_SectionPrice, 0, wxALL, 5);

	wxBoxSizer* btItem = new wxBoxSizer( wxHORIZONTAL );
	btItem->Add(new wxStaticText(this, wxID_ANY, _("Items : ")), 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	m_btAddItem = new wxButton(this, wxID_ADD);
	btItem->Add(m_btAddItem, 0, wxALL, 5);

    #if defined USE_BAR_CODE_MODE
	m_btBarCode = new wxButton(this, wxID_ANY, _("Bar Code"));
	btItem->Add(m_btBarCode, 0, wxEXPAND | wxALL, 5);
	m_btBarCode->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ScrollPanelItems::OnDialogBarCode), NULL, this);
	#endif // #if defined USE_BAR_CODE_MODE

    m_MainSizer->Add(btItem);

	SetSizer( m_MainSizer );

    Connect( wxID_ADD, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ScrollPanelItems::OnAddItem));

    #if defined USE_SCROLLED_WINDOW
	//SetRowCount(m_MainSizer->GetMinSize().GetHeight() / 10);
	SetLineCount(m_MainSizer->GetMinSize().GetHeight() / 10);
	#endif
}

//------------------------------------------------------------------------------
#if defined USE_SCROLLED_WINDOW
//wxCoord ScrollPanelItems::OnGetRowHeight(size_t row) const
wxCoord ScrollPanelItems::OnGetLineHeight(size_t line) const
{
    return 10;
}
#endif
//------------------------------------------------------------------------------
void ScrollPanelItems::ClearAll()
{
    Show(false);
    while(m_itemControls.size() != 0)
    {
        RemoveItem(0);
    }
    m_SectionPrice->SetLabel(_T(""));
    Show(true);
}
//------------------------------------------------------------------------------
#if defined USE_BAR_CODE_MODE
void ScrollPanelItems::OnDialogBarCode(wxCommandEvent& event)
{
    DialogBarCodeRead MyDialogBarCodeRead(this, wxID_ANY, m_DevisManager);
    MyDialogBarCodeRead.ShowModal();
    std::vector<std::string> &itemsRef = MyDialogBarCodeRead.GetItemsListRef();
    std::vector<double> &itemsQuantity= MyDialogBarCodeRead.GetItemsListQuantity();

    if(itemsRef.size() >= 1)
    {
        #if defined USE_SCROLLED_WINDOW
        ScrollToLine(0);
        #endif

        Show(false);

        for(int i = 0; i < itemsRef.size(); i++)
        {
            m_DevisManager->NewItem(m_posSection);
            int pos = AddItem();
            m_DevisManager->UpdateItem(m_posSection, pos, itemsRef[i], itemsQuantity[i]);
            UpdateGUIItemFromDevis(pos);
        }

        Show(true);
    }
}
#endif // #if defined USE_BAR_CODE_MODE
//------------------------------------------------------------------------------
void ScrollPanelItems::UpdateAllItemsList()
{
    m_wxAllItemsList.Clear();
    m_stdAllItemsList.clear();

    // Permet de ne generer la liste de tous les items que lors du changement de SectionPos.

    // Liste des items (STD)
    m_stdAllItemsList = m_DevisManager->GetAllItemsList();

    // Liste des items (WX + mise en forme)
    for(int i =0; i < m_stdAllItemsList.size(); i++)
    {
        m_wxAllItemsList.Add(StrStdToWx(m_stdAllItemsList[i]) + _T(" - ") + wxStrAutoCut(StrStdToWx(m_DevisManager->GetItemName(m_stdAllItemsList[i]))));
    }

}
//------------------------------------------------------------------------------
int ScrollPanelItems::AddItem()
{
    // Controls
    wxArrayString strs;

    itemControls tempItemControls;

    wxBoxSizer *sizer = new wxBoxSizer( wxHORIZONTAL );
    tempItemControls.ItemChoices = new wxChoice(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, strs);
    sizer->Add(tempItemControls.ItemChoices, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

    tempItemControls.ItemNumbers = new wxTextCtrl(this, wxID_ANY, _T(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
    sizer->Add(tempItemControls.ItemNumbers, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

    // Search
    wxImage img(20, 20);
    wxString ImgFileName = findFile(_T("data/search.png"));
    if(wxFileExists(ImgFileName))
    {
        img = wxImage(ImgFileName, wxBITMAP_TYPE_PNG);
    }
    tempItemControls.ItemSearch = new wxBitmapButton(this, wxID_ANY, wxBitmap(img));
    sizer->Add(tempItemControls.ItemSearch, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

    // Remove
    tempItemControls.ItemBtRm = new wxButton(this, wxID_ANY, _("Remove"));
    sizer->Add(tempItemControls.ItemBtRm, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

    m_MainSizer->Add(sizer, 0, wxEXPAND, 0);

    tempItemControls.ItemCaptions = new wxStaticText(this, wxID_ANY, _T(""));
    m_MainSizer->Add(tempItemControls.ItemCaptions, 0, wxLEFT, 20);

    // Events
    tempItemControls.ItemBtRm->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ScrollPanelItems::OnRemoveItem), NULL, this);
    tempItemControls.ItemChoices->Connect(  wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler(ScrollPanelItems::OnChoice), NULL, this);
    tempItemControls.ItemNumbers->Connect( wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler(ScrollPanelItems::OnChangeQuantity), NULL, this);
    tempItemControls.ItemSearch->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ScrollPanelItems::OnItemSearch), NULL, this);

    // Finition
    m_itemControls.push_back(tempItemControls);

    #if defined USE_SCROLLED_WINDOW
    RefreshAll();
    SetLineCount(m_MainSizer->GetMinSize().GetHeight() / 10);
    #endif

    m_MainSizer->Layout();

    // On retourne la pos de l'item
    return m_itemControls.size() - 1;
}

//------------------------------------------------------------------------------
void ScrollPanelItems::SetPosSection(const int &posSection)
{
    Show(false);
    UpdateAllItemsList(); // Maj de la liste des items (pour les contrôles)

    m_posSection = posSection;
    int itemNumber = m_DevisManager->GetSectionItemsNumber(m_posSection);

    // Si trop d'item actuelement ==> On enléve
    while(m_itemControls.size() > itemNumber)
    {
        RemoveItem(0);
    }
    // Si pas assez d'item actuelement ==> On ajoute
    while(m_itemControls.size() < itemNumber)
    {
        AddItem();
    }

    // Maj
    for(int pos = 0; pos < itemNumber; pos++)
    {
        UpdateGUIItemFromDevis(pos);
    }

    RefreshLabel();

    Show(true);

    /*ClearAll();

    m_posSection = posSection;

    std::vector<std::string> itemsList = m_DevisManager->GetItemsList(m_posSection);

    for(int i =0; i < itemsList.size(); i++)
    {
        AddItem(wxString(itemsList[i].c_str(), wxConvUTF8), m_DevisManager->GetItemQuantity(m_posSection, i));
    }*/
}

//------------------------------------------------------------------------------
void ScrollPanelItems::OnAddItem(wxCommandEvent& event)
{
    #if defined USE_SCROLLED_WINDOW
    //ScrollToRow(0);
    ScrollToLine(0);
    #endif

    if(m_DevisManager->GetAllItemsCount())
    {
        m_DevisManager->NewItem(m_posSection);

        Show(false);
        int pos = AddItem();
        UpdateGUIItemFromDevis(pos);
        Show(true);
    }
    else
    {
        wxMessageBox(_("There is no item in item data base !"), _("Add Item"), wxOK | wxICON_ERROR, this);
    }
}

//------------------------------------------------------------------------------
void ScrollPanelItems::OnRemoveItem(wxCommandEvent& event)
{
    int pos = -1;

    for( int i = 0; i < m_itemControls.size(); i++)
    {
        if(m_itemControls[i].ItemBtRm->GetId() == event.GetId())
        {
            pos = i;
            break;
        }
    }

    if(pos == -1)
    {
        return;
    }

    m_DevisManager->RemoveItem(m_posSection, pos);
    Show(false);
    RemoveItem(pos);
    Show(true);

    RefreshLabel();
}
//------------------------------------------------------------------------------

void ScrollPanelItems::RemoveItem(const size_t &pos)
{
    if(pos < m_itemControls.size())
    {
        m_itemControls[pos].ItemBtRm->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ScrollPanelItems::OnRemoveItem), NULL, this);
        m_itemControls[pos].ItemChoices->Disconnect(  wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler(ScrollPanelItems::OnChoice), NULL, this);
        m_itemControls[pos].ItemNumbers->Disconnect( wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler(ScrollPanelItems::OnChangeQuantity), NULL, this);
        m_itemControls[pos].ItemSearch->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(ScrollPanelItems::OnItemSearch), NULL, this);

        m_itemControls[pos].ItemChoices->Destroy();
        m_itemControls[pos].ItemNumbers->Destroy();
        m_itemControls[pos].ItemBtRm->Destroy();
        m_itemControls[pos].ItemCaptions->Destroy();
        m_itemControls[pos].ItemSearch->Destroy();

        m_itemControls.erase(m_itemControls.begin() + pos);

        #if defined USE_SCROLLED_WINDOW
        RefreshAll();

        //SetRowCount(m_MainSizer->GetMinSize().GetHeight() / 10);
        SetLineCount(m_MainSizer->GetMinSize().GetHeight() / 10);
        #endif

        m_MainSizer->Layout();
    }
}
//------------------------------------------------------------------------------
void ScrollPanelItems::EnableBt(const bool &enable)
{
    m_btAddItem->Enable(enable);
    #if defined USE_BAR_CODE_MODE
    m_btBarCode->Enable(enable);
    #endif // #if defined USE_BAR_CODE_MODE
}

//------------------------------------------------------------------------------
void ScrollPanelItems::OnChoice(wxCommandEvent& event)
{
    int pos = -1;

    for( int i = 0; i < m_itemControls.size(); i++)
    {
        if(m_itemControls[i].ItemChoices->GetId() == event.GetId())
        {
            pos = i;
            break;
        }
    }

    if(pos == -1)
    {
        return;
    }

    UpdateDevisItemFromGUI(pos);
}
//------------------------------------------------------------------------------
void ScrollPanelItems::OnItemSearch(wxCommandEvent& event)
{
    int pos = -1;

    for( int i = 0; i < m_itemControls.size(); i++)
    {
        if(m_itemControls[i].ItemSearch->GetId() == event.GetId())
        {
            pos = i;
            break;
        }
    }

    if(pos == -1)
    {
        return;
    }

    DialogItemSearch MyDialogItemSearch(this, wxID_ANY, m_DevisManager);
    MyDialogItemSearch.ShowModal();
    std::string itemRef = MyDialogItemSearch.GetItemRef();
    if(itemRef != "" && m_DevisManager->ItemExists(itemRef))
    {
        double itemQuantity = wxStrToNb<double>(m_itemControls[pos].ItemNumbers->GetValue());
        m_DevisManager->UpdateItem(m_posSection, pos, itemRef, itemQuantity);
        UpdateGUIItemFromDevis(pos);
    }
}

//------------------------------------------------------------------------------
void ScrollPanelItems::OnChangeQuantity(wxCommandEvent& event)
{
    int pos = -1;

    for( int i = 0; i < m_itemControls.size(); i++)
    {
        if(m_itemControls[i].ItemNumbers->GetId() == event.GetId())
        {
            pos = i;
            break;
        }
    }

    if(pos == -1)
    {
        return;
    }

    UpdateDevisItemFromGUI(pos);
}

//------------------------------------------------------------------------------
void ScrollPanelItems::UpdateGUIItemFromDevis(const size_t &pos)
{
    // Contenu
        // wxChoice :
            // Liste
    m_itemControls[pos].ItemChoices->Clear();
    m_itemControls[pos].ItemChoices->Append(m_wxAllItemsList);

            // Selection
    std::string stdRef = m_DevisManager->GetItemRef(m_posSection, pos);
    for(int i =0; i < m_stdAllItemsList.size(); i++)
    {
        if(stdRef == m_stdAllItemsList[i])
        {
            m_itemControls[pos].ItemChoices->SetSelection(i);
            break;
        }
    }
        // Quantity
    // ChangeValue pour ne pas déclancher d'event et faire un backup (undo / redo)
    m_itemControls[pos].ItemNumbers->ChangeValue(wxNbToStr(m_DevisManager->GetItemQuantity(m_posSection, pos)));

    // Labels
    RefreshLabel(pos);

    m_MainSizer->Layout();
}
//------------------------------------------------------------------------------
void ScrollPanelItems::UpdateDevisItemFromGUI(const size_t &pos)
{
    if(pos < m_itemControls.size())
    {
        if(m_itemControls[pos].ItemChoices->GetCurrentSelection() < m_stdAllItemsList.size()) // Si le numéro du ctrl est < au nb d'item total
        {
            double itemQuantity = wxStrToNb<double>(m_itemControls[pos].ItemNumbers->GetValue());
            m_DevisManager->UpdateItem(m_posSection, pos, m_stdAllItemsList[m_itemControls[pos].ItemChoices->GetCurrentSelection()], itemQuantity);
        }
    }
    RefreshLabel(pos);
}

//------------------------------------------------------------------------------
void ScrollPanelItems::RefreshLabel(const int &pos)
{
    if(m_DevisManager->IsOpen())
    {
        if(pos < m_itemControls.size() && pos >= 0)
        {
            std::string currentItemID = m_DevisManager->GetItemRef(m_posSection, pos);

            wxString label;
            label << wxStrAutoCut(StrStdToWx(m_DevisManager->GetItemName(currentItemID)));
            label << _T(" - ") << wxNbToStr(m_DevisManager->GetItemQuantity(m_posSection, pos));
            label << _T(" * ") << wxNbToStr(m_DevisManager->GetItemPriceHT(currentItemID));
            label << _T(" = ") << wxNbToStr(m_DevisManager->GetItemPriceWithQuantityHT(m_posSection, pos));
            m_itemControls[pos].ItemCaptions->SetLabel(label);
        }
        wxString label;
        label << _("Section price (HT) : ") << wxNbToStr(m_DevisManager->GetSectionPriceHT(m_posSection)) << _T(" ; ");
        label << _("Items number : ") << wxNbToStr(m_DevisManager->GetSectionItemsNumber(m_posSection));
        m_SectionPrice->SetLabel(label);
    }

    // On met à jour les label du main panel..
    static_cast<MainPanel*>(GetParent())->RefreshLabels();
}
